@@ -10,7 +10,7 @@ use Monitoring::Livestatus::INET;
 use Monitoring::Livestatus::UNIX;
 use Monitoring::Livestatus::MULTI;
 
-our $VERSION = '0.30';
+our $VERSION = '0.36';
 
 
 =head1 NAME
@@ -131,6 +131,7 @@ sub new {
       "use_threads"               => undef,   # use threads, default is to use threads where available
       "warnings"                  => 1,       # show warnings, for example on querys without Column: Header
       "logger"                    => undef,   # logger object used for statistical informations and errors / warnings
+      "deepcopy"                  => undef,   # copy result set to avoid errors with tied structures
     };
 
     for my $opt_key (keys %options) {
@@ -178,7 +179,9 @@ sub new {
         $self->{'peer'} = $self->{'CONNECTOR'}->{'peer'};
     }
 
-    $self->{'logger'}->debug('initialized Monitoring::Livestatus ('.$self->peer_name.')') if defined $self->{'logger'};
+    if(defined $self->{'logger'} and (!defined $self->{'backend'} or $self->{'backend'} ne 'Monitoring::Livestatus::MULTI')) {
+        $self->{'logger'}->debug('initialized Monitoring::Livestatus ('.$self->peer_name.')');
+    }
 
     return $self;
 }
@@ -897,6 +900,21 @@ useful when using multiple backends.
 
   see L<selectcol_arrayref> for more examples
 
+=head2 Deepcopy
+
+    deep copy/clone the result set.
+
+    Only effective when using multiple backends and threads.
+    This can be safely turned off if you dont change the
+    result set.
+    If you get an error like "Invalid value for shared scalar" error" this
+    should be turned on.
+
+    my $array_ref = $ml->selectcol_arrayref(
+       "GET hosts\nColumns: name contacts",
+       { Deepcopy => 1 }
+    );
+
 =head2 Rename
 
   see L<COLUMN ALIAS> for detailed explainaton
@@ -938,7 +956,7 @@ sub _send_socket {
         return('201', $self->_get_error(201), undef);
     }
 
-    $sock->read($header, 16) or return($self->_socket_error($statement, $sock, 'reading header failed: '.$!));
+    $sock->read($header, 16) or return($self->_socket_error($statement, $sock, 'reading from socket failed: '.$!));
     print "header: $header" if $self->{'verbose'};
     my($status, $msg, $content_length) = $self->_parse_header($header, $sock);
     return($status, $msg, undef) if !defined $content_length;
@@ -960,10 +978,10 @@ sub _socket_error {
     my $message = "\n";
     $message   .= "peer                ".Dumper($self->peer_name);
     $message   .= "statement           ".Dumper($statement);
-    $message   .= "socket->sockname()  ".Dumper($sock->sockname());
-    $message   .= "socket->atmark()    ".Dumper($sock->atmark());
-    $message   .= "socket->error()     ".Dumper($sock->error());
-    $message   .= "socket->timeout()   ".Dumper($sock->timeout());
+    #$message   .= "socket->sockname()  ".Dumper($sock->sockname());
+    #$message   .= "socket->atmark()    ".Dumper($sock->atmark());
+    #$message   .= "socket->error()     ".Dumper($sock->error());
+    #$message   .= "socket->timeout()   ".Dumper($sock->timeout());
     $message   .= "message             ".Dumper($body);
 
     $self->{'logger'}->error($message) if defined $self->{'logger'};
