@@ -4,6 +4,7 @@ use 5.000000;
 use strict;
 use warnings;
 use IO::Socket::INET;
+use Socket qw(IPPROTO_TCP TCP_NODELAY);
 use Carp;
 use base "Monitoring::Livestatus";
 
@@ -69,6 +70,8 @@ sub _open {
         $sock->timeout($self->{'timeout'});
     }
 
+    setsockopt($sock, IPPROTO_TCP, TCP_NODELAY, 1);
+
     return($sock);
 }
 
