@@ -6,7 +6,8 @@ use warnings;
 use Carp;
 use Data::Dumper;
 use Config;
-use Time::HiRes qw( gettimeofday tv_interval );
+use Time::HiRes qw/gettimeofday tv_interval/;
+use Scalar::Util qw/looks_like_number/;
 use Monitoring::Livestatus;
 use base "Monitoring::Livestatus";
 
@@ -93,6 +94,16 @@ sub new {
         $self->{'peer_by_key'}->{$peer->peer_key} = $peer;
     }
 
+    $self->{'allowed_options'} = {
+        'addpeer'   => 1,
+        'backend'   => 1,
+        'columns'   => 1,
+        'deepcopy'  => 1,
+        'rename'    => 1,
+        'slice'     => 1,
+        'sum'       => 1,
+    };
+
     $self->{'name'} = 'multiple connector' unless defined $self->{'name'};
     $self->{'logger'}->debug('initialized Monitoring::Livestatus::MULTI '.($self->{'use_threads'} ? 'with' : 'without' ).' threads') if defined $self->{'logger'};
 
@@ -112,13 +123,10 @@ See L<Monitoring::Livestatus> for more information.
 
 sub do {
     my $self  = shift;
-    my $opts  = $_[1];
+    my $opts  = $self->_lowercase_and_verify_options($_[1]);
     my $t0    = [gettimeofday];
 
-    # make opt hash keys lowercase
-    %{$opts} = map { lc $_ => $opts->{$_} } keys %{$opts};
-
-    $self->_do_on_peers("do", $opts->{'backend'}, @_);
+    $self->_do_on_peers("do", $opts->{'backends'}, @_);
     my $elapsed = tv_interval ( $t0 );
     $self->{'logger'}->debug(sprintf('%.4f', $elapsed).' sec for do('.$_[0].') in total') if defined $self->{'logger'};
     return 1;
@@ -134,13 +142,10 @@ See L<Monitoring::Livestatus> for more information.
 
 sub selectall_arrayref {
     my $self  = shift;
-    my $opts  = $_[1];
+    my $opts  = $self->_lowercase_and_verify_options($_[1]);
     my $t0    = [gettimeofday];
 
-    # make opt hash keys lowercase
-    %{$opts} = map { lc $_ => $opts->{$_} } keys %{$opts};
-
-    my $return  = $self->_merge_answer($self->_do_on_peers("selectall_arrayref", $opts->{'backend'}, @_));
+    my $return  = $self->_merge_answer($self->_do_on_peers("selectall_arrayref", $opts->{'backends'}, @_));
     my $elapsed = tv_interval ( $t0 );
     $self->{'logger'}->debug(sprintf('%.4f', $elapsed).' sec for selectall_arrayref() in total') if defined $self->{'logger'};
 
@@ -157,13 +162,10 @@ See L<Monitoring::Livestatus> for more information.
 
 sub selectall_hashref {
     my $self  = shift;
-    my $opts  = $_[2];
+    my $opts  = $self->_lowercase_and_verify_options($_[2]);
     my $t0    = [gettimeofday];
 
-    # make opt hash keys lowercase
-    %{$opts} = map { lc $_ => $opts->{$_} } keys %{$opts};
-
-    my $return  = $self->_merge_answer($self->_do_on_peers("selectall_hashref", $opts->{'backend'}, @_));
+    my $return  = $self->_merge_answer($self->_do_on_peers("selectall_hashref", $opts->{'backends'}, @_));
     my $elapsed = tv_interval ( $t0 );
     $self->{'logger'}->debug(sprintf('%.4f', $elapsed).' sec for selectall_hashref() in total') if defined $self->{'logger'};
 
@@ -180,13 +182,10 @@ See L<Monitoring::Livestatus> for more information.
 
 sub selectcol_arrayref {
     my $self  = shift;
-    my $opts  = $_[1];
+    my $opts  = $self->_lowercase_and_verify_options($_[1]);
     my $t0    = [gettimeofday];
 
-    # make opt hash keys lowercase
-    %{$opts} = map { lc $_ => $opts->{$_} } keys %{$opts};
-
-    my $return  = $self->_merge_answer($self->_do_on_peers("selectcol_arrayref", $opts->{'backend'}, @_));
+    my $return  = $self->_merge_answer($self->_do_on_peers("selectcol_arrayref", $opts->{'backends'}, @_));
     my $elapsed = tv_interval ( $t0 );
     $self->{'logger'}->debug(sprintf('%.4f', $elapsed).' sec for selectcol_arrayref() in total') if defined $self->{'logger'};
 
@@ -204,20 +203,17 @@ See L<Monitoring::Livestatus> for more information.
 sub selectrow_array {
     my $self      = shift;
     my $statement = $_[0];
-    my $opts      = $_[1];
+    my $opts      = $self->_lowercase_and_verify_options($_[1]);
     my $t0        = [gettimeofday];
     my @return;
 
-    # make opt hash keys lowercase
-    %{$opts} = map { lc $_ => $opts->{$_} } keys %{$opts};
-
     if(defined $opts->{'sum'} or $statement =~ m/^Stats:/mx) {
-        @return = @{$self->_sum_answer($self->_do_on_peers("selectrow_arrayref", $opts->{'backend'}, @_))};
+        @return = @{$self->_sum_answer($self->_do_on_peers("selectrow_arrayref", $opts->{'backends'}, @_))};
     } else {
         if($self->{'warnings'}) {
             carp("selectrow_arrayref without Stats on multi backend will not work as expected!");
         }
-        my $rows = $self->_merge_answer($self->_do_on_peers("selectrow_arrayref", $opts->{'backend'}, @_));
+        my $rows = $self->_merge_answer($self->_do_on_peers("selectrow_arrayref", $opts->{'backends'}, @_));
         @return = @{$rows} if defined $rows;
     }
 
@@ -238,20 +234,17 @@ See L<Monitoring::Livestatus> for more information.
 sub selectrow_arrayref {
     my $self      = shift;
     my $statement = $_[0];
-    my $opts      = $_[1];
+    my $opts      = $self->_lowercase_and_verify_options($_[1]);
     my $t0        = [gettimeofday];
     my $return;
 
-    # make opt hash keys lowercase
-    %{$opts} = map { lc $_ => $opts->{$_} } keys %{$opts};
-
     if(defined $opts->{'sum'} or $statement =~ m/^Stats:/mx) {
-        $return = $self->_sum_answer($self->_do_on_peers("selectrow_arrayref", $opts->{'backend'}, @_));
+        $return = $self->_sum_answer($self->_do_on_peers("selectrow_arrayref", $opts->{'backends'}, @_));
     } else {
         if($self->{'warnings'}) {
             carp("selectrow_arrayref without Stats on multi backend will not work as expected!");
         }
-        my $rows = $self->_merge_answer($self->_do_on_peers("selectrow_arrayref", $opts->{'backend'}, @_));
+        my $rows = $self->_merge_answer($self->_do_on_peers("selectrow_arrayref", $opts->{'backends'}, @_));
         $return = $rows->[0] if defined $rows->[0];
     }
 
@@ -272,22 +265,19 @@ See L<Monitoring::Livestatus> for more information.
 sub selectrow_hashref {
     my $self      = shift;
     my $statement = $_[0];
-    my $opts      = $_[1];
+    my $opts      = $self->_lowercase_and_verify_options($_[1]);
 
     my $t0 = [gettimeofday];
 
     my $return;
 
-    # make opt hash keys lowercase
-    %{$opts} = map { lc $_ => $opts->{$_} } keys %{$opts};
-
     if(defined $opts->{'sum'} or $statement =~ m/^Stats:/mx) {
-        $return = $self->_sum_answer($self->_do_on_peers("selectrow_hashref", $opts->{'backend'}, @_));
+        $return = $self->_sum_answer($self->_do_on_peers("selectrow_hashref", $opts->{'backends'}, @_));
     } else {
         if($self->{'warnings'}) {
             carp("selectrow_hashref without Stats on multi backend will not work as expected!");
         }
-        $return = $self->_merge_answer($self->_do_on_peers("selectrow_hashref", $opts->{'backend'}, @_));
+        $return = $self->_merge_answer($self->_do_on_peers("selectrow_hashref", $opts->{'backends'}, @_));
     }
 
     my $elapsed = tv_interval ( $t0 );
@@ -307,22 +297,19 @@ See L<Monitoring::Livestatus> for more information.
 sub selectscalar_value {
     my $self  = shift;
     my $statement = $_[0];
-    my $opts      = $_[1];
+    my $opts      = $self->_lowercase_and_verify_options($_[1]);
 
     my $t0 = [gettimeofday];
 
-    # make opt hash keys lowercase
-    %{$opts} = map { lc $_ => $opts->{$_} } keys %{$opts};
-
     my $return;
 
     if(defined $opts->{'sum'} or $statement =~ m/^Stats:/mx) {
-        return $self->_sum_answer($self->_do_on_peers("selectscalar_value", $opts->{'backend'}, @_));
+        return $self->_sum_answer($self->_do_on_peers("selectscalar_value", $opts->{'backends'}, @_));
     } else {
         if($self->{'warnings'}) {
             carp("selectscalar_value without Stats on multi backend will not work as expected!");
         }
-        my $rows = $self->_merge_answer($self->_do_on_peers("selectscalar_value", $opts->{'backend'}, @_));
+        my $rows = $self->_merge_answer($self->_do_on_peers("selectscalar_value", $opts->{'backends'}, @_));
 
         $return = $rows->[0] if defined $rows->[0];
     }
@@ -548,18 +535,23 @@ sub _do_wrapper {
 
 ########################################
 sub _do_on_peers {
-    my $self      = shift;
-    my $sub       = shift;
-    my $backends  = shift;
-    my @opts      = @_;
-    my $statement = $opts[0];
-
-    my $t0 = [gettimeofday];
+    my $self        = shift;
+    my $sub         = shift;
+    my $backends    = shift;
+    my @opts        = @_;
+    my $statement   = $opts[0];
+    my $use_threads = $self->{'use_threads'};
+    my $t0          = [gettimeofday];
 
     my $return;
     my %codes;
     my %messages;
-    my $use_threads = $self->{'use_threads'};
+    my $query_options;
+    if($sub eq 'selectall_hashref') {
+        $query_options = $self->_lowercase_and_verify_options($opts[2]);
+    } else {
+        $query_options = $self->_lowercase_and_verify_options($opts[1]);
+    }
 
     # which peers affected?
     my @peers;
@@ -589,14 +581,20 @@ sub _do_on_peers {
         # use the threaded variant
         print("using threads\n") if $self->{'verbose'};
 
-        my $x = 0;
+        my $peers_to_use;
         for my $peer (@peers) {
-            my $job = {
-                    'peer'   => $x,
-                    'sub'    => $sub,
-                    'opts'   => \@opts,
-            };
-            $self->{'WorkQueue'}->enqueue($job);
+            $peers_to_use->{$peer->peer_key} = 1;
+        }
+        my $x = 0;
+        for my $peer (@{$self->{'peers'}}) {
+            if(defined $peers_to_use->{$peer->peer_key}) {
+                my $job = {
+                        'peer'   => $x,
+                        'sub'    => $sub,
+                        'opts'   => \@opts,
+                };
+                $self->{'WorkQueue'}->enqueue($job);
+            }
             $x++;
         }
 
@@ -627,30 +625,44 @@ sub _do_on_peers {
     # check if we different result stati
     undef $Monitoring::Livestatus::ErrorMessage;
     $Monitoring::Livestatus::ErrorCode = 0;
-    my @codes = keys %codes;
+    my @codes = sort keys %codes;
     if(scalar @codes > 1) {
         # got different results for our backends
         print "got different result stati: ".Dumper(\%codes) if $self->{'verbose'};
     } else {
         # got same result codes for all backend
-        my $code = $codes[0];
-        if($code >= 300) {
-            my $msg  = $codes{$code}->[0]->{'msg'};
-            print "same: $code -> $msg\n" if $self->{'verbose'};
-            $Monitoring::Livestatus::ErrorMessage = $msg;
-            $Monitoring::Livestatus::ErrorCode    = $code;
-            if($self->{'errors_are_fatal'}) {
-                croak("ERROR ".$code." - ".$Monitoring::Livestatus::ErrorMessage." in query:\n'".$statement."'\n");
-            }
-            return;
+    }
+
+    my $failed = 0;
+    my $code = $codes[0];
+    if($code >= 300) {
+        $failed = 1;
+    }
+
+    if($failed) {
+        my $msg  = $codes{$code}->[0]->{'msg'};
+        print "same: $code -> $msg\n" if $self->{'verbose'};
+        $Monitoring::Livestatus::ErrorMessage = $msg;
+        $Monitoring::Livestatus::ErrorCode    = $code;
+        if($self->{'errors_are_fatal'}) {
+            croak("ERROR ".$code." - ".$Monitoring::Livestatus::ErrorMessage." in query:\n'".$statement."'\n");
         }
+        return;
     }
 
     my $elapsed = tv_interval ( $t0 );
     $self->{'logger'}->debug(sprintf('%.4f', $elapsed).' sec for fetching all data') if defined $self->{'logger'};
 
-    if($use_threads) {
+    # deep copy result?
+    if($use_threads
+       and (
+            (defined $query_options->{'deepcopy'} and $query_options->{'deepcopy'} == 1)
+            or
+            (defined $self->{'deepcopy'}          and $self->{'deepcopy'} == 1)
+        )
+       ) {
         # result has to be cloned to avoid "Invalid value for shared scalar" error
+
         $return = $self->_clone($return, $self->{'logger'});
     }
 
@@ -698,7 +710,7 @@ sub _sum_answer {
             for my $key (keys %{$data->{$peername}}) {
                 if(!defined $return->{$key}) {
                     $return->{$key} = $data->{$peername}->{$key};
-                } else {
+                } elsif(looks_like_number($data->{$peername}->{$key})) {
                     $return->{$key} += $data->{$peername}->{$key};
                 }
             }
@@ -770,6 +782,21 @@ sub _get_peer_by_key {
     return $self->{'peer_by_key'}->{$key};
 }
 
+########################################
+sub _lowercase_and_verify_options {
+    my $self = shift;
+    my $opts = shift;
+    my $return;
+
+    for my $key (keys %{$opts}) {
+        if($self->{'warnings'} and !defined $self->{'allowed_options'}->{lc $key}) {
+            carp("unknown option used: $key - please use only: ".join(", ", keys %{$self->{'allowed_options'}}));
+        }
+        $return->{lc $key} = $opts->{$key};
+    }
+
+    return($return);
+}
 ########################################
 
 END {
